@@ -81,7 +81,7 @@ fs_sysctl(const int name) {
 
 #define EVENT_SIZE (sizeof (struct inotify_event))
 #define EVENT_BUF_LEN (32 * (EVENT_SIZE + 16))
-#define IN_ALL IN_CLOSE_WRITE|IN_DELETE_SELF|IN_MOVE_SELF|IN_MOVED_TO|IN_MOVED_FROM|IN_ATTRIB|IN_CREATE
+#define IN_ALL IN_CLOSE_WRITE|IN_DELETE_SELF|IN_MOVE_SELF|IN_MOVE|IN_ATTRIB|IN_CREATE|IN_DELETE
 
 /*
  * inotify and kqueue ids both have the type `int`
@@ -191,6 +191,7 @@ kevent(int kq, const struct kevent *changelist, int nchanges, struct
 				if (iev->mask & IN_DELETE_SELF) fflags |= NOTE_DELETE;
 				if (iev->mask & IN_CLOSE_WRITE) fflags |= NOTE_WRITE;
 				if (iev->mask & IN_CREATE)      fflags |= NOTE_WRITE;
+				if (iev->mask & IN_DELETE)      fflags |= NOTE_WRITE;
 				if (iev->mask & IN_MOVE_SELF)   fflags |= NOTE_RENAME;
 				if (iev->mask & IN_MOVED_TO)    fflags |= NOTE_RENAME;
 				if (iev->mask & IN_MOVED_FROM)  fflags |= NOTE_RENAME;
